@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Copyright (c) 2007 Piyush verma <piyush.verma@gmail.com>                  *
  * Copyright 2007 Andreas Pakulat <apaku@gmx.de>                           *
- * Copyright 2010-2011 Sven Brauch <svenbrauch@googlemail.com>               *
+ * Copyright 2010-2012 Sven Brauch <svenbrauch@googlemail.com>               *
  *                                                                           *
  * Permission is hereby granted, free of charge, to any person obtaining     *
  * a copy of this software and associated documentation files (the           *
@@ -22,6 +22,19 @@
  * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION     *
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.           *
  *****************************************************************************/
+
+#include "pythonlanguagesupport.h"
+#include "pythoneditorintegrator.h"
+#include "dumpchain.h"
+#include "usebuilder.h"
+#include "contextbuilder.h"
+#include "pythonducontext.h"
+#include "pythonparsejob.h"
+#include "declarationbuilder.h"
+#include "helpers.h"
+
+#include <KStandardDirs>
+
 #include <ktexteditor/smartrange.h>
 #include <ktexteditor/smartinterface.h>
 #include <ktexteditor/document.h>
@@ -38,18 +51,6 @@
 #include <interfaces/iproject.h>
 #include <project/projectmodel.h>
 
-#include <KStandardDirs>
-
-#include "pythonlanguagesupport.h"
-#include "pythoneditorintegrator.h"
-#include "dumpchain.h"
-#include "usebuilder.h"
-#include "contextbuilder.h"
-#include "pythonducontext.h"
-#include "pythonparsejob.h"
-#include "declarationbuilder.h"
-#include "helpers.h"
-
 using namespace KDevelop;
 
 using namespace KTextEditor;
@@ -314,10 +315,10 @@ void ContextBuilder::visitCode(CodeAst* node) {
         }
         
         if ( ! internal ) {
-            // If the built-in functions file is not yet parsed, schedule it with a high priority and abort.
-            m_unresolvedImports.append(doc_url);
+            // If the built-in functions file is not yet parsed, schedule it with a high priority.
+            m_unresolvedImports.append(doc);
             KDevelop::ICore::self()->languageController()->backgroundParser()
-                                   ->addDocument(doc_url, KDevelop::TopDUContext::ForceUpdate,
+                                   ->addDocument(doc, KDevelop::TopDUContext::ForceUpdate,
                                                  BackgroundParser::BestPriority, 0, ParseJob::FullSequentialProcessing);
             // This must NOT be called from parse threads! It's only meant to be used from the foreground thread, and will
             // cause thread starvation if called from here.
@@ -464,8 +465,14 @@ void ContextBuilder::visitFunctionBody(FunctionDefinitionAst* node)
 {
     // The function should end at the next DEDENT token, not at the body's last statement
     int endLine = node->endLine;
+    if ( ! node->body.isEmpty() ) {
+        endLine = node->body.last()->startLine;
+    }
     if ( node->endLine != node->startLine ) {
         endLine = editor()->indent()->nextChange(endLine, FileIndentInformation::Dedent);
+        if ( ! node->body.isEmpty() ) {
+            endLine = qMax<int>(endLine, node->body.last()->endLine + 1);
+        }
     }
     CursorInRevision end = CursorInRevision(endLine, node->startLine == node->endLine ? INT_MAX : 0);
     CursorInRevision start = rangeForArgumentsContext(node).end;
