@@ -23,67 +23,60 @@
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.           *
  *****************************************************************************/
 #include "pythonparsejob.h"
-#include <QFile>
-#include <QThread>
-#include <QReadLocker>
-
-#include <ktexteditor/document.h>
-#include <ktexteditor/smartinterface.h>
-
-#include <kdebug.h>
-#include <klocale.h>
-
-#include <language/duchain/duchainlock.h>
-#include <language/duchain/duchain.h>
-#include <language/duchain/topducontext.h>
-#include <language/duchain/dumpdotgraph.h>
-#include <interfaces/ilanguage.h>
-#include <language/backgroundparser/urlparselock.h>
 
 #include "pythonhighlighting.h"
 #include "pythoneditorintegrator.h"
 #include "dumpchain.h"
 #include "parsesession.h"
 #include "pythonlanguagesupport.h"
-// #include "contextbuilder.h"
 #include "declarationbuilder.h"
 #include "usebuilder.h"
-#include <interfaces/foregroundlock.h>
-// #include "astprinter.h"
-// #include "usebuilder.h"
-#include <language/highlighting/codehighlighting.h>
-#include <interfaces/icore.h>
-#include <interfaces/ilanguagecontroller.h>
+
+#include <language/duchain/duchainlock.h>
+#include <language/duchain/duchain.h>
+#include <language/duchain/topducontext.h>
+#include <language/duchain/dumpdotgraph.h>
 #include <language/duchain/indexedstring.h>
 #include <language/duchain/duchainutils.h>
+#include <language/backgroundparser/urlparselock.h>
 #include <language/backgroundparser/backgroundparser.h>
+#include <language/highlighting/codehighlighting.h>
 #include <language/interfaces/iastcontainer.h>
+#include <interfaces/ilanguage.h>
+#include <interfaces/foregroundlock.h>
+#include <interfaces/icore.h>
+#include <interfaces/ilanguagecontroller.h>
+#include <interfaces/idocumentcontroller.h>
+
+#include <ktexteditor/document.h>
+#include <ktexteditor/smartinterface.h>
+
+#include <QReadLocker>
+#include <QFile>
+#include <QThread>
+#include <QProcess>
+#include <QTemporaryFile>
+#include <kdebug.h>
+#include <klocale.h>
+#include <KConfigGroup>
 
 using namespace KDevelop;
 
 namespace Python
 {
 
-ParseJob::ParseJob(LanguageSupport* parent, const KUrl &url)
-        : KDevelop::ParseJob(url)
+ParseJob::ParseJob(const IndexedString &url, ILanguageSupport* languageSupport)
+        : KDevelop::ParseJob(url, languageSupport)
         , m_ast(0)
         , m_readFromDisk(false)
         , m_duContext(0)
-        , m_url(url)
 {
-    m_parent = parent;
 }
 
 ParseJob::~ParseJob()
 {
 }
 
-LanguageSupport *ParseJob::python() const
-{
-    return LanguageSupport::self();
-}
-
-
 CodeAst *ParseJob::ast() const
 {
     Q_ASSERT( isFinished() && m_ast );
@@ -95,35 +88,19 @@ bool ParseJob::wasReadFromDisk() const
     return m_readFromDisk;
 }
 
-void loadImportsFromDisk(TopDUContext* context) {
-    QVector< DUContext::Import > imports = context->importedParentContexts();
-    foreach ( const KDevelop::DUContext::Import& i, imports ) {
-        if ( TopDUContextPointer p = TopDUContextPointer(dynamic_cast<TopDUContext*>(i.context(context))) ) {
-            loadImportsFromDisk(p.data());
-        }
-    }
-}
-
 void ParseJob::run()
 {
     ParseSession* currentSession = new ParseSession();
-    LanguageSupport* lang = python();
-    ILanguage* ilang = lang->language();
     
     if ( abortRequested() || ICore::self()->shuttingDown() ) {
         delete currentSession;
         return abortJob();
     }
     
-    qDebug() << " ====> PARSING ====> parsing file " << m_url.path() << "; has priority" << parsePriority();
-    
-    if ( ! lang || ! ilang ) {
-        kWarning() << "Language support is NULL";
-        return abortJob();
-    }
+    qDebug() << " ====> PARSING ====> parsing file " << document().toUrl() << "; has priority" << parsePriority();
     
     // lock the URL so no other parse job can run on this document
-    QReadLocker parselock(ilang->parseLock());
+    QReadLocker parselock(languageSupport()->language()->parseLock());
     UrlParseLock urlLock(document());
     
     readContents();
@@ -131,18 +108,16 @@ void ParseJob::run()
     if ( !(minimumFeatures() & TopDUContext::ForceUpdate || minimumFeatures() & Rescheduled) ) {
         DUChainReadLocker lock(DUChain::lock());
         static const IndexedString langString("python");
-        foreach(ParsingEnvironmentFilePointer file, DUChain::self()->allEnvironmentFiles(document())) {
+        foreach(const ParsingEnvironmentFilePointer &file, DUChain::self()->allEnvironmentFiles(document())) {
             if ( file->language() != langString ) {
                 continue;
             }
             if ( ! file->needsUpdate() && file->featuresSatisfied(minimumFeatures()) && file->topContext() ) {
                 qDebug() << " ====> NOOP    ====> Already up to date:" << document().str();
                 setDuChain(file->topContext());
-                loadImportsFromDisk(duChain());
                 if ( ICore::self()->languageController()->backgroundParser()->trackerForUrl(document()) ) {
                     lock.unlock();
-                    KDevelop::ICodeHighlighting* hl = m_parent->codeHighlighting();
-                    hl->highlightDUChain(duChain());
+                    highlightDUChain();
                 }
                 delete currentSession;
                 return;
@@ -162,9 +137,7 @@ void ParseJob::run()
     }
     
     currentSession->setContents(QString::fromUtf8(contents().contents));
-    currentSession->setCurrentDocument(m_url);
-    
-    IndexedString filename = KDevelop::IndexedString(m_url.pathOrUrl());
+    currentSession->setCurrentDocument(document());
     
     // call the python API and the AST transformer to populate the syntax tree
     QPair<CodeAst*, bool> parserResults = currentSession->parse(m_ast);
@@ -184,11 +157,11 @@ void ParseJob::run()
         // set up the declaration builder, it gets the parsePriority so it can re-schedule imported files with a better priority
         DeclarationBuilder builder(editor.data());
         builder.m_ownPriority = parsePriority();
-        builder.m_currentlyParsedDocument = filename;
+        builder.m_currentlyParsedDocument = document();
         builder.m_futureModificationRevision = contents().modification;
-        
+
         // Run the declaration builder. If necessary, it will run itself again.
-        m_duContext = builder.build(filename, m_ast, toUpdate.data());
+        m_duContext = builder.build(document(), m_ast, toUpdate.data());
         if ( abortRequested() ) {
             delete currentSession;
             return abortJob();
@@ -198,7 +171,7 @@ void ParseJob::run()
         
         // gather uses of variables and functions on the document
         UseBuilder usebuilder(editor.data());
-        usebuilder.m_currentlyParsedDocument = filename;
+        usebuilder.m_currentlyParsedDocument = document();
         usebuilder.buildUses(m_ast);
         
         // check whether any unresolved imports were encountered
@@ -208,8 +181,8 @@ void ParseJob::run()
             // check whether one of the imports is queued for parsing, this is to avoid deadlocks
             // it's also ok if the duchain is now available (and thus has been parsed before already)
             bool dependencyInQueue = false;
-            DUChainWriteLocker lock;
-            foreach ( const KUrl& url, builder.m_unresolvedImports ) {
+            DUChainWriteLocker lock(DUChain::lock());
+            foreach ( const IndexedString& url, builder.m_unresolvedImports ) {
                 dependencyInQueue = KDevelop::ICore::self()->languageController()->backgroundParser()->isQueued(url);
                 dependencyInQueue = dependencyInQueue || DUChain::self()->chainForDocument(url);
                 if ( dependencyInQueue ) {
@@ -220,7 +193,7 @@ void ParseJob::run()
             // this prevents infinite loops in case something goes wrong (optimally, shouldn't reach here if
             // the document was already rescheduled, but there's many cases where this might still happen)
             if ( ! ( minimumFeatures() & Rescheduled ) && dependencyInQueue ) {
-                KDevelop::ICore::self()->languageController()->backgroundParser()->addDocument(document().toUrl(), 
+                KDevelop::ICore::self()->languageController()->backgroundParser()->addDocument(document(),
                                      static_cast<TopDUContext::Features>(TopDUContext::ForceUpdate | Rescheduled), parsePriority(),
                                      0, ParseJob::FullSequentialProcessing);
             }
@@ -235,7 +208,7 @@ void ParseJob::run()
             DUChain::self()->updateContextEnvironment(m_duContext, parsingEnvironmentFile.data());
         }
         
-        kDebug() << "---- Parsing Succeded ----";
+        kDebug() << "---- Parsing Succeeded ----";
         
         if ( abortRequested() ) {
             delete currentSession;
@@ -243,11 +216,7 @@ void ParseJob::run()
         }
         
         // start the code highlighter if parsing was successful.
-        if ( m_parent && m_parent->codeHighlighting() ) {
-            kDebug() << "Starting highlighter...";
-            KDevelop::ICodeHighlighting* hl = m_parent->codeHighlighting();
-            hl->highlightDUChain(m_duContext);
-        }
+        highlightDUChain();
     }
     else {
         // No syntax tree was received from the parser, the expected reason for this is a syntax error in the document.
@@ -283,9 +252,12 @@ void ParseJob::run()
     
     // The parser might have given us some syntax errors, which are now added to the document.
     DUChainWriteLocker lock(DUChain::lock());
-    foreach ( ProblemPointer p, currentSession->m_problems ) {
+    foreach ( const ProblemPointer& p, currentSession->m_problems ) {
         m_duContext->addProblem(p);
     }
+
+    // If enabled, and if the document is open, do PEP8 checking.
+    eventuallyDoPEP8Checking(document(), m_duContext);
     
     if ( minimumFeatures() & TopDUContext::AST ) {
         DUChainWriteLocker lock;
@@ -299,6 +271,85 @@ void ParseJob::run()
     setDuChain(m_duContext);
 }
 
+void ParseJob::eventuallyDoPEP8Checking(const IndexedString document, TopDUContext* topContext)
+{
+    IDocument* idoc = ICore::self()->documentController()->documentForUrl(document.toUrl());
+    if ( ! idoc || ! topContext || topContext->features() & PEP8Checking ) {
+        return;
+    }
+
+    KConfig config("kdevpythonsupportrc");
+    KConfigGroup configGroup = config.group("pep8");
+    if ( ! configGroup.readEntry<bool>("pep8enabled", false) ) {
+        return;
+    }
+    {
+        DUChainWriteLocker lock;
+        topContext->setFeatures((TopDUContext::Features) ( topContext->features() | PEP8Checking ));
+    }
+    kDebug() << "doing pep8 checking";
+    // TODO that's not very elegant, better would be making pep8 read from stdin -- but it doesn't support that atm
+    QTemporaryFile tempfile;
+    tempfile.open();
+    tempfile.write(idoc->textDocument()->text().toUtf8());
+    tempfile.close();
+    QString url = configGroup.readEntry("pep8url", "/usr/bin/pep8-python2");
+    QFileInfo f(url);
+    bool error = false;
+    if ( ! f.isExecutable() ) {
+        error = true;
+    }
+    QProcess process;
+    process.setProcessChannelMode(QProcess::MergedChannels);
+    process.start(url, QStringList() << tempfile.fileName());
+    process.waitForFinished(1000);
+    if ( process.state() != QProcess::NotRunning || ( process.exitCode() != 0 && process.exitCode() != 1 )  ) {
+        process.kill();
+        error = true;
+    }
+    else {
+        QByteArray data = process.readAll();
+        QList<QByteArray> errors = data.split('\n');
+        QRegExp errorFormat("(.*):(\\d*):(\\d*): (.*)", Qt::CaseInsensitive, QRegExp::RegExp2);
+        DUChainWriteLocker lock;
+        foreach ( const QByteArray& error, errors ) {
+            if ( errorFormat.exactMatch(error.data()) ) {
+                const QStringList texts = errorFormat.capturedTexts();
+                bool lineno_ok = false;
+                bool colno_ok = false;
+                int lineno = texts.at(2).toInt(&lineno_ok);
+                int colno = texts.at(3).toInt(&colno_ok);
+                if ( ! lineno_ok || ! colno_ok ) {
+                    kDebug() << "invalid line / col number:" << texts;
+                    continue;
+                }
+                QString error = texts.at(4);
+                KDevelop::Problem *p = new KDevelop::Problem();
+                p->setFinalLocation(DocumentRange(document, SimpleRange(lineno - 1, qMax(colno - 4, 0),
+                                                                        lineno - 1, colno + 4)));
+                p->setSource(KDevelop::ProblemData::Preprocessor);
+                p->setSeverity(KDevelop::ProblemData::Warning);
+                p->setDescription(i18n("PEP8 checker error: %1", error));
+                ProblemPointer ptr(p);
+                topContext->addProblem(ptr);
+            }
+            else {
+                kDebug() << "invalid pep8 error line:" << error;
+            }
+        }
+    }
+    if ( error ) {
+        DUChainWriteLocker lock;
+        KDevelop::Problem *p = new KDevelop::Problem();
+        p->setFinalLocation(DocumentRange(document, SimpleRange(0, 0, 0, 0)));
+        p->setSource(KDevelop::ProblemData::Preprocessor);
+        p->setSeverity(KDevelop::ProblemData::Warning);
+        p->setDescription(i18n("The selected PEP8 syntax checker \"%1\" does not seem to work correctly.", url));
+        ProblemPointer ptr(p);
+        topContext->addProblem(ptr);
+    }
+}
+
 }
 
 #include "pythonparsejob.moc"
